/* This source file is part of the Procept Operating System */
 
/*This file is prepared for Doxygen automatic documentation generation.*/
 
/*! @file *********************************************************************
 *
 * @brief The header file for pack.c
 *
 * - Compiler:           uVision5 for nRF51
 * - Supported devices:  nRF51822xxAA
 * - AppNote:
 *
 * @author               tchinnadurai
 ******************************************************************************/
 
/* Copyright (c) 2011 Procept IP. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted unless the following conditions are met.
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of Procept may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with a Procept
 * Product with permission of the Procept Board of Directors.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCEPT "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 **/

#ifndef PACK_H__
#define PACK_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble_nus.h"
#include "nrf_delay.h"
#define POLY 0x8408
#define UART_TX_TIMEOUT	20//5000//20//20msec
#define MAX_RETRY_COUNT	3
#define NUM_PBUFF_ENTRIES	6//2//3//10//todo test this
#define NUM_OF_USED_BYTES	10////2bytes for index, 1 byte for ID, 2bytes for CRC, 4 bytes for inuse,ispacked,issent,retrycount,sender
#define MAX_DATA_LEN	BLE_NUS_MAX_DATA_LEN-NUM_OF_USED_BYTES
#define RX_ACK	0x01
#define RX_NACK	0x02
#define TX_ACK	0x01
#define TX_NACK	0x02
#define RPM_ID	0x1E
#define PERIODIC_STATUS_TX_TO_NRF	1000//milliseconds
#define PERIODIC_COMMAND_TX_TO_STM	20//milliseconds
#define	SENDER_NRF_COMMAND	0x01
#define	SENDER_NRF_RESPONSE	0x02
#define	SENDER_STM32_COMMAND	0x03
#define	SENDER_STM32_RESPONSE	0x04
/* Forward declaration of the pack_buffer_t type. */
typedef struct pack_buffer_s pack_buffer_t;
/**@brief Pack buffer structure.
 *
 * @details This structure contains all information related to packing data and sending out via uart.
 */
struct pack_buffer_s
{
    uint16_t                 index;               		//incremental number
    uint16_t                 id;          						//The ID of the command that is being sent from remote to each of the wheels
		uint8_t						 			 sender;									//Sender is 01:CommandFromNRF;02:ResponseFromNRF;03:CommandFromSTM32;04:ResponseFromSTM32	
    uint8_t 								 data[MAX_DATA_LEN]; //2bytes for index, 1 byte for ID, 2bytes for CRC, 4 bytes for inuse,ispacked,issent,retrycount
    uint16_t								 crc; 										//2 byte crc
		uint8_t 								 inUse;										//inUse is 1 if the data has not been sent or, 
																											//sent successfully. It is 0, if correct ACK has been received.
		uint8_t 								 isPacked;								//isPacked is 1 if crc field has been populated and is for ready to
																											//be sent via uart because it is packed now.
		uint8_t 								 isSent;									//isSent is 1 if data has been sent and is waiting for ACK/NACK
		uint8_t 								 retryCount;							//retryCount is incremented every time the packed data has been sent out via uart.
		uint32_t 								 timerIntrptCount;				//Used to find if timeout has occurred.
};
/**@brief Connection handle statuses.
 */
typedef enum
{
    SUCCESS=0,       		//Operation resulted in success.
    NO_EMPTY_ENTRIES,  //The command corresponding to previous button 
											 //press at remote was not saved at wheel 
											 //because the receive buffer is full at the wheel.
		CRC_MISMATCH, 		//CRC mimatch occurred between received and calculated values
		OUT_OF_RANGE=0xFFFF
} pack_status_t;
void pack_init(void);
uint16_t pack_writePopulateTxBuffer(uint8_t input);
uint16_t pack_getReadTxBufferIndex(void);
void pack_readPopulateTxBuffer(uint16_t index);
uint16_t pack_getWriteTxBufferIndex(void);
void pack_incrementReadTxBufferIndex(void);
void pack_resetReadTxBufferIndex(void);
void pack_packInBufferAndSendViaUart(uint16_t index);
void pack_clearBuffer(uint16_t index);
uint16_t pack_incrementRetryCountAndResend(uint16_t index);
void pack_checkIfTimeoutHappened(void);
uint16_t pack_VerifyRxCRC(uint8_t *rxedBuffer);
uint16_t pack_calculateRxCRC(uint8_t *buffer);
void pack_sendResponseToSTM32(uint8_t * buffer, uint8_t ack_or_nack);
uint16_t pack_findCrcToDecideNumberOfBytesToRead(uint8_t *rxedBuffer, uint32_t number_of_received_bytes_minus_crc);
void pack_sendCommandFromNRFtoSTM(void);
void error_occurred(void);
#endif // PACK_H__
/** @} */
