/* This source file is part of the Procept Operating System */
 
/*This file is prepared for Doxygen automatic documentation generation.*/
 
/*! @file *********************************************************************
 *
 * @brief The implementation of the functionalities needed 
 *        for handling the UART module.
 *
 * - Compiler:           uVision5 for nRF51
 * - Supported devices:  nRF51822xxAA
 * - AppNote:
 *
 * @author               tchinnadurai
 ******************************************************************************/
 
/* Copyright (c) 2011 Procept IP. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted unless the following conditions are met.
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of Procept may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with a Procept
 * Product with permission of the Procept Board of Directors.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCEPT "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 **/

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include "pins.h"
#include "nrf_gpio.h"
//#include "boards.h"
//#include "ble_dtm.h"
#include "nrf_drv_gpiote.h"
//#include "flash.h"
//#include "timer.h"
#include "uart.h"

//#include "adc.h"
//#include "flash.h"
//#include "led.h"
#include "app_error.h"
#include "nrf_uart.h"
//#include "program.h"
//#include "charge_generator.h"
//#include "error.h"

static void uart0PrintAsAsciiSmall(uint32_t result);
static uint8_t AppInfoPrint = 0;
static uint8_t * printStatement=(uint8_t *)"\r\nUART\r\n";
static volatile uint32_t ReceivedData = 0;
static volatile uint32_t completedFlushing=0;
static volatile uint32_t receivedDataReady = 0;
static volatile uint32_t read_ctr = 0;
static volatile uint8_t byte_0 =0;
static volatile uint8_t byte_1 =0;
static volatile uint8_t rx_data[MAX_TEST_DATA_BYTES];
static volatile uint32_t debug_tx_ctr =0;
static volatile uint32_t number_of_received_bytes =0;
static volatile uint8_t valid_sequence =0;
/************************************************
 * 				PUBLIC ROUTINES					*
 ************************************************/


uint32_t uart_getDataReceived(void)
{
	return ReceivedData;
}
void uart_clearDataReceived(void)
{
	ReceivedData = 0;
}
void uart_setDataReceived(void)
{
	ReceivedData = 1;
}
uint32_t uart_getReceiveDataReady(void)
{
	return receivedDataReady;
}
void uart_clearReceiveDataReady (void)
{
	receivedDataReady = 0;
}
void uart_setReceiveDataReady (void)
{
	receivedDataReady = 1;
}
void uart_getReceivedBytes(uint8_t* data_buffer)
{
	for(uint32_t i = 0; i< number_of_received_bytes; i++)
	{
		*data_buffer++ = rx_data[i];
	}
}
void uart_receiveBuffer(void)
{
		if((uart_getDataReceived()!=0)&&(valid_sequence==1))
		{
			uart_clearDataReceived();
			if(read_ctr>4)//2 bytes for preamble, 2 bytes for crc, at least 1 byte for data
			{
				number_of_received_bytes=read_ctr;

				uint16_t calculated_crc= pack_findCrcToDecideNumberOfBytesToRead(&rx_data[0],number_of_received_bytes-2);//-2 because last 2 bytes for crc
				//from this for debugging
				if(number_of_received_bytes==MAX_RECEIVED_DATA_BYTES)
				{
					__NOP;
					valid_sequence=0;
				}
				//till this for debugging
				uint16_t received_crc=rx_data[number_of_received_bytes-2]<<8;
				received_crc|=rx_data[number_of_received_bytes-1];
				if(calculated_crc==received_crc)
				{
					read_ctr=0;
					valid_sequence=0;
					uart_setReceiveDataReady();					
				}
				else if(number_of_received_bytes>=MAX_RECEIVED_DATA_BYTES)
				{
					read_ctr=0;			
					valid_sequence=0;
				}
			}

		}			
}
#define PREAMBLE_MSB 0xAB
#define PREAMBLE_LSB 0xCD
uint8_t preamble_ms_rxed =0;
void uart_error_handle(app_uart_evt_t * p_event)
{
		uint32_t err_code;	
		if(completedFlushing!=0)
		{
			if (p_event->evt_type == APP_UART_TX_EMPTY)
			{
			__NOP();
			}
			if (p_event->evt_type == APP_UART_DATA_READY)
			{
			__NOP();
			}			
		}	
	
    if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_communication);
    }
    else if (p_event->evt_type == APP_UART_FIFO_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_code);
    }
    else if ((p_event->evt_type == APP_UART_DATA_READY)||(p_event->evt_type == APP_UART_DATA))
    {
        uart_setDataReceived();//received
			err_code = app_uart_get(&rx_data[read_ctr++]);
			if(err_code!=0)
			{
				__NOP;
			}
			if(	err_code	==0)
			{
				if(preamble_ms_rxed==1)
				{
					if(rx_data[read_ctr-1]==PREAMBLE_LSB)
					{
						read_ctr=2;
						rx_data[0]=PREAMBLE_MSB;
						rx_data[1]=PREAMBLE_LSB;
						valid_sequence=1;
					}
					else
					{
						preamble_ms_rxed=0;
					}
				}
				
				if(rx_data[read_ctr-1]==PREAMBLE_MSB)
				{
					preamble_ms_rxed=1;
				}
			}
			//uart_receiveBuffer();
					NRF_UART0->EVENTS_RXDRDY = 0;
    }	
    else if (p_event->evt_type == APP_UART_TX_EMPTY)
    {
        __NOP();//txed
			debug_tx_ctr++;
					NRF_UART0->EVENTS_TXDRDY = 0;
    }	
		//nrf_uart_int_enable(NRF_UART0, NRF_UART_INT_MASK_RXDRDY | NRF_UART_INT_MASK_ERROR);
}
void uart_clearTxCtr(void)
{
	debug_tx_ctr=0;
}
uint32_t uart_getTxCtr(void)
{
	return debug_tx_ctr;
}
/** @brief Function for setting the @ref ERROR_PIN high, and then enter an infinite loop.
 */
static void show_error(void)
{
    
    //LEDS_ON(LEDS_MASK);
    while(true)
    {
        // Do nothing.
    }
}


#define RX_PIN_NUMBER_UART  6//11
#define TX_PIN_NUMBER_UART  5//9
#define CTS_PIN_NUMBER_UART 11//10
#define RTS_PIN_NUMBER_UART 11//8

void uart_wIntInit(void)
{

    nrf_gpio_cfg_output(TX_PIN_NUMBER_UART);
    nrf_gpio_cfg_input(RX_PIN_NUMBER_UART, NRF_GPIO_PIN_NOPULL);
    
uint32_t err_code;
    const app_uart_comm_params_t comm_params =
      {
          RX_PIN_NUMBER_UART,
          TX_PIN_NUMBER_UART,
          RTS_PIN_NUMBER_UART,
          CTS_PIN_NUMBER_UART,
          APP_UART_FLOW_CONTROL_ENABLED,
          false,
          UART_BAUDRATE_BAUDRATE_Baud115200
      };

    APP_UART_FIFO_INIT(&comm_params,
                         UART_RX_BUF_SIZE,
                         UART_TX_BUF_SIZE,
                         uart_error_handle,
                         APP_IRQ_PRIORITY_LOWEST,
                         err_code);

    APP_ERROR_CHECK(err_code);				
}

/************************************************
 * 				PRIVATE ROUTINES					*
 ************************************************/



/**
 * @}
 */
