/* This source file is part of the Procept Operating System */
 
/*This file is prepared for Doxygen automatic documentation generation.*/
 
/*! @file *********************************************************************
 *
 * @brief The header file for uart.c
 *
 * - Compiler:           uVision5 for nRF51
 * - Supported devices:  nRF51822xxAA
 * - AppNote:
 *
 * @author               tchinnadurai
 ******************************************************************************/
 
/* Copyright (c) 2011 Procept IP. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted unless the following conditions are met.
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of Procept may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with a Procept
 * Product with permission of the Procept Board of Directors.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCEPT "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 **/

#ifndef UART_H__
#define UART_H__

#include <stdint.h>
#include <stdbool.h>
#include "app_uart.h"
//#include "pins.h"
#define BITRATE  UART_BAUDRATE_BAUDRATE_Baud115200  /**< Serial bitrate on the UART */
#define PIN_UART0_TXD												5
#define PIN_UART0_RXD												6//7	//6 for C.golf board	
#define RTS_PIN_NUMBER	0	//dummy
#define CTS_PIN_NUMBER	0 //dummy

#define MAX_TEST_DATA_BYTES     256//(30U)                /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE MAX_TEST_DATA_BYTES                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE MAX_TEST_DATA_BYTES                           /**< UART RX buffer size. */
#define RTS_PIN_NUMBER	0
#define CTS_PIN_NUMBER	0
#define MAX_RECEIVED_DATA_BYTES		256//(16U)//(6U)//2bytes for index, 1 byte for ack/nack, 2 bytes for crc
#define QUIET_TIME_UART_TRANSMIT_MS	5//in milliseconds
typedef enum
{
    eBUTTON   							 							= 0,           
    eLED_STANDBY			        			 			= 1,           
    eBLE								   	 							= 2,         
    eADC									  						  = 3,           
    eSTATEMC_OFF		   			 							= 4,          
    eSTATEMC_STANDBY   			 							= 5, 	
    eSTATEMC_FIRST_TIME_PAIRING   			 	= 6, 
    eSTATEMC_PAIRING   			 							= 7, 
    eSTATEMC_ACTIVE   			 							= 8, 
    eSTATEMC_PAUSE   			 								= 9, 	
    eSTATEMC_CHANGE_SYSTEM_RF_MODE   			= 10, 	
    eFLASH			      			 							= 11,
    eLED_OFF					        			 			= 12, 
    eESB							        			 			= 13, 	
    eTMR							        			 			= 14, 
    ePROGRAM   							 							= 15, 	
    eCHARGE_GENERATOR				 							= 16, 	
		eLED_FIRST_TIME_PAIRING 							= 17, 
		eLED_RF_CHANGE_SYSTEM_MODE						= 18 
}enumTestIDPrefix;

typedef enum
{
    eInfoNotPresent   			 							= 0,           
    eInfoPresent			        			 			= 1
}enumTestIDInfo;

typedef enum
{
    eFail   				 							= 0,           
    ePass			        			 			= 1,           
    eUserFill		   	 							= 2 
}enumTestIDResult;

/*!
*	@brief 	Function for the UART0 initialization.
*	@param[in] None. 
*	@return None.
*/
void uart_init(void);
void uart_wIntInit(void);

/**
 * @brief Prints the input Sentence
 *
* @param[in] 					input: input sentence
 * @returns             None
 */
void uart0TxWriteSentence(uint8_t * input);

/**
 * @brief Prints the test output with id, prefix, pass or fail or userfil and additional info
 *
 * @param[in] 					testID: test number
 * @param[in] 					testIdPrefix: prefix for each test
 * @param[in] 					passFailInput: pass/fail/userfil
 * @param[in] 					infoPresent: Information that needs to be printed
 * @returns             None
 */
void uart_printTestOutput(uint32_t testID,enumTestIDPrefix testIdPrefix, enumTestIDResult passFailInput, enumTestIDInfo infoPresent);

/**
 * @brief Prints the input number which is less than 100
 *
* @param[in] 					testId: input number
 * @returns             None
 */
void uart_printSmallNumber(uint32_t testId );


/**
 * @brief Prints the heading in the uart terminal.
 *
* @param[in] 					version: input version number
 * @returns             None
 */
void uart_printHeading(uint32_t version);


/**
 * @brief If this routine is called, then the smart device info
 *        is printed in the heading.
 *
* @param[in] 					None
 * @returns             None
 */
void uart_setAppInfoPrint(void);

/**
 * @brief If this routine is called, then the smart device info
 *        is not printed in the heading.
 *
* @param[in] 					None
 * @returns             None
 */
void uart_clearAppInfoPrint(void);

/**
 * @brief Prints the end of Sentence
 *
* @param[in] 					None
 * @returns             None
 */
void uart_printEnd(void);
void uart0Tx(uint8_t input);
void uart0Tx_Byte(uint8_t input);
void uart_receiveBuffer(void);
uint32_t uart_getReceiveDataReady(void);
void uart_getReceivedBytes(uint8_t* data_buffer);
void uart_clearReceiveDataReady (void);
#endif // UART_H__
/** @} */
