/* This source file is part of the Procept Operating System */
 
/*This file is prepared for Doxygen automatic documentation generation.*/
 
/*! @file *********************************************************************
 *
 * @brief The implementation of the functionalities needed 
 *        for handling the UART module.
 *
 * - Compiler:           uVision5 for nRF51
 * - Supported devices:  nRF51822xxAA
 * - AppNote:
 *
 * @author               tchinnadurai
 ******************************************************************************/
 
/* Copyright (c) 2011 Procept IP. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted unless the following conditions are met.
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of Procept may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with a Procept
 * Product with permission of the Procept Board of Directors.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCEPT "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 **/

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include "pins.h"
#include "nrf_gpio.h"
//#include "boards.h"
//#include "ble_dtm.h"
#include "nrf_drv_gpiote.h"
//#include "flash.h"
//#include "timer.h"
#include "uart.h"

//#include "adc.h"
//#include "flash.h"
//#include "led.h"
#include "app_error.h"
//#include "program.h"
//#include "charge_generator.h"
//#include "error.h"

static void uart0PrintAsAsciiSmall(uint32_t result);
static uint8_t AppInfoPrint = 0;
static uint8_t * printStatement=(uint8_t *)"\r\nUART\r\n";
static volatile uint32_t ReceivedData = 0;
static volatile uint32_t completedFlushing=0;
static volatile uint32_t receivedDataReady = 0;
static volatile uint32_t read_ctr = 0;
static volatile uint8_t byte_0 =0;
static volatile uint8_t byte_1 =0;
static volatile uint8_t rx_data[MAX_TEST_DATA_BYTES];
uint32_t debugCtr= 0;
static volatile uint32_t debug_tx_ctr=0;
static volatile uint32_t number_of_received_bytes =0;
/************************************************
 * 				PUBLIC ROUTINES					*
 ************************************************/
/*!
*	@brief 	Function for the UART initialization.
*	@param[in] None. 
*	@return None.
*/
void uart_init(void)
{
    // Configure UART0 pins.
    nrf_gpio_cfg_output(PIN_UART0_TXD);
    nrf_gpio_cfg_input(PIN_UART0_RXD, NRF_GPIO_PIN_NOPULL);

    NRF_UART0->PSELTXD       = PIN_UART0_TXD;
    NRF_UART0->PSELRXD       = PIN_UART0_RXD;
    NRF_UART0->BAUDRATE      = BITRATE;

    // Clean out possible events from earlier operations
    NRF_UART0->EVENTS_RXDRDY = 0;
    NRF_UART0->EVENTS_TXDRDY = 0;
    NRF_UART0->EVENTS_ERROR  = 0;

    // Activate UART.
    NRF_UART0->ENABLE        = UART_ENABLE_ENABLE_Enabled;
    NRF_UART0->INTENSET      = 0;
    NRF_UART0->TASKS_STARTTX = 1;
    NRF_UART0->TASKS_STARTRX = 1;
}

uint32_t uart_getDataReceived(void)
{
	return ReceivedData;
}
void uart_clearDataReceived(void)
{
	ReceivedData = 0;
}
void uart_setDataReceived(void)
{
	ReceivedData = 1;
}
uint32_t uart_getReceiveDataReady(void)
{
	return receivedDataReady;
}
void uart_clearReceiveDataReady (void)
{
	receivedDataReady = 0;
}
void uart_setReceiveDataReady (void)
{
	receivedDataReady = 1;
}
void uart_getReceivedBytes(uint8_t* data_buffer)
{
	for(uint32_t i = 0; i< number_of_received_bytes; i++)
	{
		*data_buffer++ = rx_data[i];
	}
}

void uart_receiveBuffer(void)
{
  uint32_t err_code;
	/*if(1)//(uart_getReceiveDataReady()==0)
	{	
		if(uart_getDataReceived()!=0)
		{
			uart_clearDataReceived();

			err_code = app_uart_get(&rx_data[read_ctr++]);
			if(err_code!=0)
			{
				__NOP;
			}
		}
//		if(read_ctr==MAX_RECEIVED_DATA_BYTES)
		{
			read_ctr=0;
			uart_setReceiveDataReady();
		}	
	}*/
		if(uart_getDataReceived()!=0)
		{
			uart_clearDataReceived();
			if(read_ctr>2)
			{
				number_of_received_bytes=read_ctr;

				uint16_t calculated_crc= pack_findCrcToDecideNumberOfBytesToRead(&rx_data[0],number_of_received_bytes-2);//-2 because last 2 bytes for crc
				//from this for debugging
				if(number_of_received_bytes==MAX_RECEIVED_DATA_BYTES)
				{
					__NOP;
				}
				//till this for debugging
				uint16_t received_crc=rx_data[number_of_received_bytes-2]<<8;
				received_crc|=rx_data[number_of_received_bytes-1];
				if(calculated_crc==received_crc)
				{
					read_ctr=0;
					uart_setReceiveDataReady();					
				}
			}

		}	
}

 uint8_t uart0Rx(void)
{
	uint8_t rx_byte=0;
	for (;;)
	{
		if (NRF_UART0->EVENTS_RXDRDY == 0)
		{
				// Nothing read from the UART.
				continue;
		}
		NRF_UART0->EVENTS_RXDRDY = 0;
		rx_byte  = (uint8_t)NRF_UART0->RXD;	
		break;
	}
	return rx_byte;
}

void uart_receiveBuffer2(void)
{
	read_ctr=0;
	for(int i=0;i<MAX_RECEIVED_DATA_BYTES;i++)
	{
		rx_data[read_ctr++]=uart0Rx();
	}
	__NOP;
}
/*
void uart_receiveBuffer(void)
{
  uint32_t err_code;
//	for(int i=0;i<MAX_RECEIVED_DATA_BYTES;i++)//if(1)//(uart_getReceiveDataReady()==0)
	{	
		if(uart_getDataReceived()!=0)
		{
			uart_clearDataReceived();
			do
			{
				err_code = app_uart_get(&rx_data[read_ctr++]);
				if(err_code!=0)
				{
					__NOP;
				}
			}while(err_code!=0);
		}
	}
//	if(read_ctr==MAX_RECEIVED_DATA_BYTES)
	{
		read_ctr=0;
		uart_setReceiveDataReady();
	}		
}*/
void uart_error_handle(app_uart_evt_t * p_event)
{
		if(completedFlushing!=0)
		{
			if (p_event->evt_type == APP_UART_TX_EMPTY)
			{
			__NOP();
			}
			if (p_event->evt_type == APP_UART_DATA_READY)
			{
			__NOP();
			}			
		}	
	
    if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_communication);
    }
    else if (p_event->evt_type == APP_UART_FIFO_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_code);
    }
    else if ((p_event->evt_type == APP_UART_DATA_READY)||(p_event->evt_type == APP_UART_DATA))
    {
        uart_setDataReceived();//received
			uint32_t err_code = app_uart_get(&rx_data[read_ctr++]);
			if(err_code!=0)
			{
				__NOP;
			}					
			//uart_receiveBuffer();			
					NRF_UART0->EVENTS_RXDRDY = 0;
		/*	debugCtr++;
			if(debugCtr>14)
			{
				__NOP;
			}
			*/
    }	
    else if (p_event->evt_type == APP_UART_TX_EMPTY)
    {
        __NOP();//txed
			debug_tx_ctr++;
					NRF_UART0->EVENTS_TXDRDY = 0;
    }	
}
void uart_clearTxCtr(void)
{
	debug_tx_ctr=0;
}
uint32_t uart_getTxCtr(void)
{
	return debug_tx_ctr;
}
/** @brief Function for setting the @ref ERROR_PIN high, and then enter an infinite loop.
 */
static void show_error(void)
{
    
    //LEDS_ON(LEDS_MASK);
    while(true)
    {
        // Do nothing.
    }
}


/** @brief Function for testing UART loop back. 
 *  @details Transmitts one character at a time to check if the data received from the loopback is same as the transmitted data.
 *  @note  @ref TX_PIN_NUMBER must be connected to @ref RX_PIN_NUMBER)
 */
static void uart_loopback_test()
{
    uint8_t * tx_data = (uint8_t *)("\n\rHELLO\n\r");
    uint8_t   rx_data;
		uint32_t err_code;
    // Start sending one byte and see if you get the same
    for (uint32_t i = 0; i < MAX_TEST_DATA_BYTES; i++)
    {
        
			rx_data=0;
       // while(app_uart_put(tx_data[i]) != NRF_SUCCESS);

        //nrf_delay_ms(10);
        err_code = app_uart_get(&rx_data);
			if(rx_data==0x33)
			{
				__NOP();
			}
			else if(rx_data==34)
			{
				__NOP();
			}

        if ((rx_data != tx_data[i]) || (err_code != NRF_SUCCESS))
        {
            //show_error();
        }
    }
   // for (uint32_t i = 0; i < MAX_TEST_DATA_BYTES; i++)
    //{		
				if(uart_getDataReceived()!=0)
				{
					uart_clearDataReceived();
					err_code = app_uart_get(&rx_data);
				//	byte_0=app_uart_get_received_0();
				//	byte_1=app_uart_get_received_1();
					while(app_uart_put(byte_0) != NRF_SUCCESS);
					__NOP();
					app_uart_flush();
					completedFlushing=1;
				}	
	//	}				
    return;
}

void uart_wIntInit(void)
{
    uint32_t err_code;
    const app_uart_comm_params_t comm_params =
      {
          PIN_UART0_RXD,
          PIN_UART0_TXD,
          RTS_PIN_NUMBER,
          CTS_PIN_NUMBER,
          APP_UART_FLOW_CONTROL_ENABLED,
          false,
          BITRATE
      };

    APP_UART_FIFO_INIT(&comm_params,
                         UART_RX_BUF_SIZE,
                         UART_TX_BUF_SIZE,
                         uart_error_handle,
                         APP_IRQ_PRIORITY_LOW,
                         err_code);

    APP_ERROR_CHECK(err_code);	
}

/**
 * @brief Prints the input Sentence
 *
* @param[in] 					input: input sentence
 * @returns             None
 */
void uart0TxWriteSentence(uint8_t * input)
{
	uint32_t count=0;
	do{
		count++;
	}while(input[count]!=0x00);
	for(int i=0;i<count+1;i++)
	{
		uart0Tx(input[i]);
	}		
}


void uart_printTime(void)
{
	double timeInSeconds = 0;//timer_getRtc1Count()/(1000/timer_getRTC1Count());//convert to 1 second equivalent
	double timeInMinutes = timeInSeconds/60.0;
	double timeInHours = timeInMinutes/60.0;

	uint32_t timeToDisplayInDays = timeInHours/24;
	uint32_t timeToDisplayInHours = (((double) (timeInHours/24.0)) - timeToDisplayInDays)*24;
	uint32_t timeToDisplayInMinutes = (((double) (timeInMinutes/60.0)) - (uint32_t)(timeInMinutes/60))*60;
	uint32_t timeToDisplayInSeconds = ((double)((double) (timeInSeconds/60.0)) - ((uint32_t)(timeInSeconds/60)))*60;
	
	uart0PrintAsAsciiSmall(timeToDisplayInDays);
	uart0Tx(':');
	uart0PrintAsAsciiSmall(timeToDisplayInHours);
	uart0Tx(':');
	uart0PrintAsAsciiSmall(timeToDisplayInMinutes);
	uart0Tx(':');
	uart0PrintAsAsciiSmall(timeToDisplayInSeconds);
	uart0Tx(':');
}

void uart_printTestIDPrefix(enumTestIDPrefix testIdPrefix )
{
	if(testIdPrefix==eBUTTON)
	{
		printStatement=(uint8_t *)"BTN";
		uart0TxWriteSentence(printStatement);			
	}
	else if(testIdPrefix==ePROGRAM)
	{
		printStatement=(uint8_t *)"PRG";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eCHARGE_GENERATOR)
	{
		printStatement=(uint8_t *)"CHG";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eLED_STANDBY)
	{
		printStatement=(uint8_t *)"LST";
		uart0TxWriteSentence(printStatement);			
	}
	else if(testIdPrefix==eLED_OFF)
	{
		printStatement=(uint8_t *)"LOF";
		uart0TxWriteSentence(printStatement);			
	}
	else if(testIdPrefix==eLED_RF_CHANGE_SYSTEM_MODE)
	{
		printStatement=(uint8_t *)"LRF";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eLED_FIRST_TIME_PAIRING)
	{
		printStatement=(uint8_t *)"LFP";
		uart0TxWriteSentence(printStatement);			
	}		
	else if(testIdPrefix==eTMR)
	{
		printStatement=(uint8_t *)"TMR";
		uart0TxWriteSentence(printStatement);			
	}		
	else if(testIdPrefix==eBLE)
	{
		printStatement=(uint8_t *)"BLE";
		uart0TxWriteSentence(printStatement);			
	}
	else if(testIdPrefix==eADC)
	{
		printStatement=(uint8_t *)"ADC";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eSTATEMC_STANDBY)
	{
		printStatement=(uint8_t *)"STS";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eSTATEMC_OFF)
	{
		printStatement=(uint8_t *)"STO";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eSTATEMC_FIRST_TIME_PAIRING)
	{
		printStatement=(uint8_t *)"STF";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eSTATEMC_PAIRING)
	{
		printStatement=(uint8_t *)"STP";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eSTATEMC_ACTIVE)
	{
  	printStatement=(uint8_t *)"STA";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eSTATEMC_PAUSE)
	{
		printStatement=(uint8_t *)"STU";
		uart0TxWriteSentence(printStatement);			
	}	
	else if(testIdPrefix==eSTATEMC_CHANGE_SYSTEM_RF_MODE)
	{
		printStatement=(uint8_t *)"STR";
		uart0TxWriteSentence(printStatement);				
	}	
	else if(testIdPrefix==eESB)
	{
		printStatement=(uint8_t *)"ESB";
		uart0TxWriteSentence(printStatement);			
	}		
	else if(testIdPrefix==eFLASH)
	{
		printStatement=(uint8_t *)"FSH";
		uart0TxWriteSentence(printStatement);			
	}		
}
/**
 * @brief Prints the input number which is less than 100
 *
* @param[in] 					testId: input number
 * @returns             None
 */
void uart_printSmallNumber(uint32_t testId )
{
	uart0PrintAsAsciiSmall(testId);
}

void uart_printFail(void)
{
		printStatement=(uint8_t *)"FAIL\r\n";
		uart0TxWriteSentence(printStatement);		
}

void uart_printPass(void)
{
		printStatement=(uint8_t *)"PASS\r\n";
		uart0TxWriteSentence(printStatement);		
}

void uart_printUserFil(void)
{
		printStatement=(uint8_t *)"USERFILL\r\n";
		uart0TxWriteSentence(printStatement);		
}
/**
 * @brief Prints either pass or fail or user fill column
 *
* @param[in] 					None
 * @returns             None
 */
void uart_printPassOrFail(uint32_t Input )
{
	if(Input==0)
	{
		//print FAIL
		printStatement=(uint8_t *)"FAIL  ";
		uart0TxWriteSentence(printStatement);			
	}
	else if(Input==1)
	{
		//print PASS
		printStatement=(uint8_t *)"PASS  ";
		uart0TxWriteSentence(printStatement);			
	}
	else if(Input==2)
	{
		//print User fills
		printStatement=(uint8_t *)"USERFILL";
		uart0TxWriteSentence(printStatement);		
	}	
	
	uart0Tx(':');		
}

/**
 * @brief If this routine is called, then the smart device info
 *        is printed in the heading.
 *
* @param[in] 					None
 * @returns             None
 */
void uart_setAppInfoPrint(void)
{
	AppInfoPrint = 1;
}
/**
 * @brief If this routine is called, then the smart device info
 *        is not printed in the heading.
 *
* @param[in] 					None
 * @returns             None
 */
void uart_clearAppInfoPrint(void)
{
	AppInfoPrint = 0;
}
/**
 * @brief Prints the end of Sentence
 *
* @param[in] 					None
 * @returns             None
 */
void uart_printEnd(void)
{
	printStatement=(uint8_t *)"\r\n";
	uart0TxWriteSentence(printStatement);	
}	
/**
 * @brief Prints the heading in the uart terminal.
 *
* @param[in] 					version: input version number
 * @returns             None
 */
void uart_printHeading(uint32_t version)
{
	printStatement=(uint8_t *)"\r\n";
	uart0TxWriteSentence(printStatement);		

	//To Print (example)
	//Tester Name: Tara Chinnadurai
	//Tester email ID and Name	tchinnadurai@procept.com.au
	//Date of testing	9/12/2016
	//Hardware Version	v0.4
	//Firmware Version	0.1.0.0
	//MAC address of DUT	C:54:33:1B:8E:A5
	//Test set up:	Unless specified, all tests were done using power 
	//supply voltage: 3.01V and 500mA current limit with 500 ohms load 
	//connected across the Plus and Minus electrodes.
	//Equipments used	1) BK Precision DC Power supply (PTM000109)
	//2) DS4012 Digital Oscilloscope(PTM000064)
	//3) Samsung Galaxy S6 Android Version: 5.1.1
	//4) iOS (iPAD) version 9.3.4 (13G35)
	
	printStatement=(uint8_t *)"Tester Name    				:";
	uart0TxWriteSentence(printStatement);		
	printStatement=(uint8_t *)"Tara Chinnadurai\r\n";
	uart0TxWriteSentence(printStatement);	

	printStatement=(uint8_t *)"Tester email ID				:";
	uart0TxWriteSentence(printStatement);		
	printStatement=(uint8_t *)"tchinnadurai@procept.com.au\r\n";
	uart0TxWriteSentence(printStatement);		
	
	printStatement=(uint8_t *)"Date of testing				:";
	uart0TxWriteSentence(printStatement);		
	printStatement=(uint8_t *)"14-12-2016\r\n";
	uart0TxWriteSentence(printStatement);		
	
	printStatement=(uint8_t *)"Hardware Version			:";
	uart0TxWriteSentence(printStatement);		
	printStatement=(uint8_t *)"v0.5\r\n";
	uart0TxWriteSentence(printStatement);		
	
	printStatement=(uint8_t *)"Firmware Version			:v";
	uart0TxWriteSentence(printStatement);
	uint32_t upperversion = 	version >>24;
	uint32_t middleversion = 	version >>16 & 0xFF;
	uint32_t lowerversion = 	version >>8 & 0xFF;
	uart_printSmallNumber(upperversion);
	uart0Tx(':');
	uart_printSmallNumber(middleversion);
	uart0Tx(':');
	uart_printSmallNumber(lowerversion);
	
	printStatement=(uint8_t *)"\r\n";
	uart0TxWriteSentence(printStatement);		

	printStatement=(uint8_t *)"MAC Addr        			:";
	uart0TxWriteSentence(printStatement);		
	printStatement=(uint8_t *)"DE:65:14:23:32:2C\r\n";
	uart0TxWriteSentence(printStatement);
	
	printStatement=(uint8_t *)"Test Set-up     			:";
	uart0TxWriteSentence(printStatement);		
	printStatement=(uint8_t *)"Used power supply with 3.01V and 500mA current limit with 500 ohms load";
	uart0TxWriteSentence(printStatement);	
	printStatement=(uint8_t *)"connected across the Plus and Minus electrodes(P3 and P4).\r\n";
	uart0TxWriteSentence(printStatement);		

	printStatement=(uint8_t *)"Equipments used 			:";
	uart0TxWriteSentence(printStatement);		
	printStatement=(uint8_t *)"BK Precision DC Power supply (PTM000109),";
	uart0TxWriteSentence(printStatement);	
	printStatement=(uint8_t *)"DS4012 Digital Oscilloscope(PTM000064),";
	uart0TxWriteSentence(printStatement);	
	if(AppInfoPrint)
	{
		printStatement=(uint8_t *)"Samsung Galaxy S6 Android Version: 5.1.1,";
		uart0TxWriteSentence(printStatement);	
		printStatement=(uint8_t *)"iOS (iPAD) version 9.3.4 (13G35)";
		uart0TxWriteSentence(printStatement);	
	}		
	printStatement=(uint8_t *)"\r\n";
	uart0TxWriteSentence(printStatement);
	
	printStatement=(uint8_t *)"DAY:HRS:MIN:SEC:  ID  :RESULT:INFO\r\n";
	uart0TxWriteSentence(printStatement);
}
/**
 * @brief Prints the test output with id, prefix, pass or fail or userfil and additional info
 *
 * @param[in] 					testID: test number
 * @param[in] 					testIdPrefix: prefix for each test
 * @param[in] 					passFailInput: pass/fail/userfil
 * @param[in] 					infoPresent: Information that needs to be printed
 * @returns             None
 */

void uart_printTestOutput(uint32_t testID,enumTestIDPrefix testIdPrefix, enumTestIDResult passFailInput, enumTestIDInfo infoPresent)
{
	uart_printTime();
	uart_printTestIDPrefix(testIdPrefix);
	uart_printSmallNumber(testID);
	uart0Tx(':');	
	
	if(infoPresent==eInfoNotPresent)
	{
		if(passFailInput==eFail)
		{
			uart_printFail();
		}
		else if(passFailInput==ePass)
		{
			uart_printPass();
		}
		else if(passFailInput==eUserFill)
		{
			uart_printUserFil();
		}		
	}
	else
	{
		uart_printPassOrFail(passFailInput);
	}
}

/************************************************
 * 				PRIVATE ROUTINES					*
 ************************************************/

/**
 * @brief Transmits bytes via uart
 *
 * @param[in] 					Byte to be transmitted.
 * @returns             None
 */
void uart0Tx(uint8_t input)
{
				NRF_UART0->TXD = input;
				// Wait until MSB is sent.
				while (NRF_UART0->EVENTS_TXDRDY != 1)
				{
						// Do nothing.
				}
				NRF_UART0->EVENTS_TXDRDY = 0;	
}

void uart0Tx_Byte(uint8_t input)
{
				NRF_UART0->TXD = input;
				// Wait until MSB is sent.
				while (NRF_UART0->EVENTS_TXDRDY != 1)
				{
						// Do nothing.
				}
				NRF_UART0->EVENTS_TXDRDY = 0;	
}
/**
 * @brief Prints ascii characters to uart for numbers less than 100.
 *
 * @param[in] 					input to be printed.
 * @returns             None
 */
static void uart0PrintAsAsciiSmall(uint32_t result)
{
	uint16_t result3=0;
	if(result>=100)
	{
		result3 = (result/100) + 0x30;
		result =result - ((result/100) * 100);
	}
	else
	{
		result3 = 0x30;
	}					
	uint16_t result4=0;
	if(result>=10)
	{
		result4 = (result/10) + 0x30;
		result =result - ((result/10) * 10);
	}
	else
	{
		result4 = 0x30;
	}
	uint16_t result5=0;
	if(result>=1)
	{
		result5 = (result/1) + 0x30;
		result =result - ((result/1) * 1);
	}
	else
	{
		result5 = 0x30;
	}		
	uart0Tx(((uint8_t)(result3)));
	uart0Tx(((uint8_t)(result4)));
	uart0Tx(((uint8_t)(result5)));	
}

/**
 * @}
 */
